%
%     This file is part of CasADi.
%
%     CasADi -- A symbolic framework for dynamic optimization.
%     Copyright (C) 2010-2014 Joel Andersson, Joris Gillis, Moritz Diehl,
%                             K.U. Leuven. All rights reserved.
%     Copyright (C) 2011-2014 Greg Horn
%
%     CasADi is free software; you can redistribute it and/or
%     modify it under the terms of the GNU Lesser General Public
%     License as published by the Free Software Foundation; either
%     version 3 of the License, or (at your option) any later version.
%
%     CasADi is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%     Lesser General Public License for more details.
%
%     You should have received a copy of the GNU Lesser General Public
%     License along with CasADi; if not, write to the Free Software
%     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
%
%
import casadi.*

%Solve the Rosenbrock problem, formulated as the NLP:
%
%minimize     x^2 + 100*z^2
%subject to   z+(1-x)^2-y == 0
%
%Joel Andersson, 2015

% Declare variables
x = SX.sym('x');
y = SX.sym('y');
z = SX.sym('z');

% Formulate the NLP
disp('== Solving constrained optimization problem ==')
f = x^2 + 100*z^2;
g = z + (1-x)^2 - y;
nlp = struct('x', [x;y;z], 'f', f, 'g', g);

% Specify solver options
opts = struct;
%opts.ipopt.hessian_approximation = 'limited-memory';

% Create an NLP solver
<<<<<<< HEAD:exercises/week1/exercise2/code/solution_ex2_new.m
solver = nlpsol('solver', 'ipopt', nlp, opts);
=======
opts = struct;
opts.ipopt.hessian_approximation = 'limited-memory';
solver = nlpsol('solver', 'ipopt', nlp,opts);
>>>>>>> 816ef3d548f3fb328c6cf428cfae0b8744c24596:exercises/week1/exercise2/code/solution/main.m

% Solve the problem
res = solver(struct('x0', [2.5,3.0,0.75], 'ubg', 0, 'lbg', ...
    0, 'ubx', inf, 'lbx', -inf));

% Print solution
disp('Optimal cost')
disp(full(res.f));
disp('Primal solution')
disp(full(res.x));
disp('Dual solution (simple bounds)')
disp(full(res.lam_x));
disp('Dual solution (nonlinear bounds)')
disp(full(res.lam_g));

% Formulate the unconstrained optimization problem
disp('== Solving unconstrained optimization problem ==')
f = x^2 + 100*(y-(1-x)^2)^2;
nlp = struct('x', [x;y], 'f', f);
 
% Create an NLP solver
solver = nlpsol('solver', 'ipopt', nlp);

% Solve the problem
res = solver(struct('x0', [2.5,3.0]));

% Print solution
disp('Optimal cost')
disp(full(res.f));
disp('Primal solution')
disp(full(res.x));
disp('Dual solution (simple bounds)')
disp(full(res.lam_x));
disp('Dual solution (nonlinear bounds)')
disp(full(res.lam_g));

% Formulate the corresponding rootfinding problem
disp(char(10))
disp('== Solving rootfinding problem ==')
res = gradient(f, [x;y]);
rfp = Function('rfp', {[x;y]}, {res}, char('x0'), char('x'));
G = rootfinder('G', 'newton', rfp, struct('print_iteration', true));

% Solve the problem
res = G(struct('x0', [2.5,3.0]));
disp('Rootfinding problem solution)')
disp(full(res.x));
